# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools
from unittest import mock

from senlinclient import plugin
from senlinclient.v1 import client


@mock.patch.object(plugin, 'create_connection')
class ClientTest(testtools.TestCase):

    def setUp(self):
        super(ClientTest, self).setUp()
        self.conn = mock.Mock()
        self.service = mock.Mock()
        self.conn.cluster = self.service

    def test_init_default(self, mock_conn):
        mock_conn.return_value = self.conn

        sc = client.Client()

        self.assertEqual(self.conn, sc.conn)
        self.assertEqual(self.service, sc.service)
        mock_conn.assert_called_once_with(prof=None, user_agent=None)

    def test_init_with_params(self, mock_conn):
        mock_conn.return_value = self.conn

        sc = client.Client(prof='FOO', user_agent='BAR', zoo='LARR')

        self.assertEqual(self.conn, sc.conn)
        self.assertEqual(self.service, sc.service)
        mock_conn.assert_called_once_with(prof='FOO', user_agent='BAR',
                                          zoo='LARR')

    def test_profile_types(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.profile_types(foo='bar')
        self.assertEqual(self.service.profile_types.return_value, res)
        self.service.profile_types.assert_called_once_with(foo='bar')

    def test_get_profile_type(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_profile_type('FOOBAR')
        self.assertEqual(self.service.get_profile_type.return_value, res)
        self.service.get_profile_type.assert_called_once_with('FOOBAR')

    def test_profiles(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.profiles(foo='bar')
        self.assertEqual(self.service.profiles.return_value, res)
        self.service.profiles.assert_called_once_with(foo='bar')

    def test_get_profile(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_profile('FOOBAR')
        self.assertEqual(self.service.get_profile.return_value, res)
        self.service.get_profile.assert_called_once_with('FOOBAR')

    def test_update_profile(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.update_profile('FAKE_ID', foo='bar')
        self.assertEqual(self.service.update_profile.return_value, res)
        self.service.update_profile.assert_called_once_with('FAKE_ID',
                                                            foo='bar')

    def test_delete_profile(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_profile('FAKE_ID')
        self.assertEqual(self.service.delete_profile.return_value, res)
        self.service.delete_profile.assert_called_once_with(
            'FAKE_ID', True)

    def test_delete_profile_ignore_missing(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_profile('FAKE_ID', False)
        self.assertEqual(self.service.delete_profile.return_value, res)
        self.service.delete_profile.assert_called_once_with(
            'FAKE_ID', False)

    def test_policy_types(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.policy_types(foo='bar')
        self.assertEqual(self.service.policy_types.return_value, res)
        self.service.policy_types.assert_called_once_with(foo='bar')

    def test_get_policy_type(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_policy_type('FOOBAR')
        self.assertEqual(self.service.get_policy_type.return_value, res)
        self.service.get_policy_type.assert_called_once_with('FOOBAR')

    def test_policies(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.policies(foo='bar')
        self.assertEqual(self.service.policies.return_value, res)
        self.service.policies.assert_called_once_with(foo='bar')

    def test_get_policy(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_policy('FOOBAR')
        self.assertEqual(self.service.get_policy.return_value, res)
        self.service.get_policy.assert_called_once_with('FOOBAR')

    def test_update_policy(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.update_policy('FAKE_ID', foo='bar')
        self.assertEqual(self.service.update_policy.return_value, res)
        self.service.update_policy.assert_called_once_with(
            'FAKE_ID', foo='bar')

    def test_delete_policy(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_policy('FAKE_ID')
        self.assertEqual(self.service.delete_policy.return_value, res)
        self.service.delete_policy.assert_called_once_with(
            'FAKE_ID', True)

    def test_delete_policy_ignore_missing(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_policy('FAKE_ID', False)
        self.assertEqual(self.service.delete_policy.return_value, res)
        self.service.delete_policy.assert_called_once_with(
            'FAKE_ID', False)

    def test_clusters(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.clusters(foo='bar')
        self.assertEqual(self.service.clusters.return_value, res)
        self.service.clusters.assert_called_once_with(foo='bar')

    def test_get_cluster(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_cluster('FOOBAR')
        self.assertEqual(self.service.get_cluster.return_value, res)
        self.service.get_cluster.assert_called_once_with('FOOBAR')

    def test_create_cluster(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.create_cluster(name='FOO', bar='zoo')
        self.assertEqual(self.service.create_cluster.return_value, res)
        self.service.create_cluster.assert_called_once_with(
            name='FOO', bar='zoo')

    def test_update_cluster(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.update_cluster('FAKE_ID', foo='bar')
        self.assertEqual(self.service.update_cluster.return_value, res)
        self.service.update_cluster.assert_called_once_with(
            'FAKE_ID', foo='bar')

    def test_delete_cluster(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_cluster('FAKE_ID', True)
        self.assertEqual(self.service.delete_cluster.return_value, res)
        self.service.delete_cluster.assert_called_once_with(
            'FAKE_ID', True, False)

    def test_delete_cluster_ignore_missing(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_cluster('FAKE_ID', True, False)
        self.assertEqual(self.service.delete_cluster.return_value, res)
        self.service.delete_cluster.assert_called_once_with(
            'FAKE_ID', True, False)

    def test_cluster_add_nodes(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_add_nodes('FAKE_ID', ['NODE1', 'NODE2'])
        self.assertEqual(self.service.add_nodes_to_cluster.return_value, res)
        self.service.add_nodes_to_cluster.assert_called_once_with(
            'FAKE_ID', ['NODE1', 'NODE2'])

    def test_cluster_del_nodes(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_del_nodes('FAKE_ID', ['NODE1', 'NODE2'])
        self.assertEqual(self.service.remove_nodes_from_cluster.return_value,
                         res)
        self.service.remove_nodes_from_cluster.assert_called_once_with(
            'FAKE_ID', ['NODE1', 'NODE2'])

    def test_cluster_resize(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_resize('FAKE_ID', foo='bar', zoo=1)
        self.assertEqual(self.service.resize_cluster.return_value, res)
        self.service.resize_cluster.assert_called_once_with(
            'FAKE_ID', foo='bar', zoo=1)

    def test_cluster_scale_in(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_scale_in('FAKE_ID', 3)
        self.assertEqual(self.service.scale_in_cluster.return_value, res)
        self.service.scale_in_cluster.assert_called_once_with(
            'FAKE_ID', 3)

    def test_cluster_scale_out(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_scale_out('FAKE_ID', 3)
        self.assertEqual(self.service.scale_out_cluster.return_value, res)
        self.service.scale_out_cluster.assert_called_once_with(
            'FAKE_ID', 3)

    def test_cluster_policies(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_policies('CLUSTER', foo='bar')
        self.assertEqual(self.service.cluster_policies.return_value, res)
        self.service.cluster_policies.assert_called_once_with(
            'CLUSTER', foo='bar')

    def test_get_cluster_policy(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_cluster_policy('PID', 'CID')
        self.assertEqual(self.service.get_cluster_policy.return_value, res)
        self.service.get_cluster_policy.assert_called_once_with(
            'PID', 'CID')

    def test_cluster_attach_policy(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_attach_policy('FOO', 'BAR', zoo='car')
        self.assertEqual(self.service.attach_policy_to_cluster.return_value,
                         res)
        self.service.attach_policy_to_cluster.assert_called_once_with(
            'FOO', 'BAR', zoo='car')

    def test_cluster_detach_policy(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_detach_policy('FOO', 'BAR')
        self.assertEqual(self.service.detach_policy_from_cluster.return_value,
                         res)
        self.service.detach_policy_from_cluster.assert_called_once_with(
            'FOO', 'BAR')

    def test_cluster_update_policy(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.cluster_update_policy('FOO', 'BAR', foo='bar')
        self.assertEqual(self.service.update_cluster_policy.return_value, res)
        self.service.update_cluster_policy.assert_called_once_with(
            'FOO', 'BAR', foo='bar')

    def test_check_cluster(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.check_cluster('FAKE_CLUSTER_ID')
        self.assertEqual(self.service.check_cluster.return_value, res)
        self.service.check_cluster.assert_called_once_with('FAKE_CLUSTER_ID')

    def test_recover_cluster(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.recover_cluster('FAKE_CLUSTER_ID')
        self.assertEqual(self.service.recover_cluster.return_value, res)
        self.service.recover_cluster.assert_called_once_with(
            'FAKE_CLUSTER_ID')

    def test_nodes(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.nodes(foo='bar')
        self.assertEqual(self.service.nodes.return_value, res)
        self.service.nodes.assert_called_once_with(foo='bar')

    def test_get_node(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_node('FOOBAR')
        self.assertEqual(self.service.get_node.return_value, res)
        self.service.get_node.assert_called_once_with('FOOBAR', details=False)

    def test_get_node_with_details(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_node('FOOBAR', details=True)
        self.assertEqual(self.service.get_node.return_value, res)
        self.service.get_node.assert_called_once_with(
            'FOOBAR', details=True)

    def test_create_node(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.create_node(name='FAKE_NAME', foo='bar')
        self.assertEqual(self.service.create_node.return_value, res)
        self.service.create_node.assert_called_once_with(
            name='FAKE_NAME', foo='bar')

    def test_update_node(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.update_node('FAKE_ID', foo='bar')
        self.assertEqual(self.service.update_node.return_value, res)
        self.service.update_node.assert_called_once_with(
            'FAKE_ID', foo='bar')

    def test_delete_node(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_node('FAKE_ID', True)
        self.assertEqual(self.service.delete_node.return_value, res)
        self.service.delete_node.assert_called_once_with(
            'FAKE_ID', True, False)

    def test_check_node(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.check_node('FAKE_ID')
        self.assertEqual(self.service.check_node.return_value, res)
        self.service.check_node.assert_called_once_with('FAKE_ID')

    def test_recover_node(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.recover_node('FAKE_ID')
        self.assertEqual(self.service.recover_node.return_value, res)
        self.service.recover_node.assert_called_once_with(
            'FAKE_ID')

    def test_delete_node_ignore_missing(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_node('FAKE_ID', True, False)
        self.assertEqual(self.service.delete_node.return_value, res)
        self.service.delete_node.assert_called_once_with(
            'FAKE_ID', True, False)

    def test_receivers(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.receivers(foo='bar')
        self.assertEqual(self.service.receivers.return_value, res)
        self.service.receivers.assert_called_once_with(foo='bar')

    def test_get_receiver(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_receiver('FOOBAR')
        self.assertEqual(self.service.get_receiver.return_value, res)
        self.service.get_receiver.assert_called_once_with('FOOBAR')

    def test_create_receiver(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.create_receiver(name='FAKE_NAME', foo='bar')
        self.assertEqual(self.service.create_receiver.return_value, res)
        self.service.create_receiver.assert_called_once_with(
            name='FAKE_NAME', foo='bar')

    def test_delete_receiver(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_receiver('FAKE_ID')
        self.assertEqual(self.service.delete_receiver.return_value, res)
        self.service.delete_receiver.assert_called_once_with(
            'FAKE_ID', True)

    def test_delete_receiver_ignore_missing(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.delete_receiver('FAKE_ID', False)
        self.assertEqual(self.service.delete_receiver.return_value, res)
        self.service.delete_receiver.assert_called_once_with(
            'FAKE_ID', False)

    def test_actions(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.actions(foo='bar')
        self.assertEqual(self.service.actions.return_value, res)
        self.service.actions.assert_called_once_with(foo='bar')

    def test_get_action(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_action('FOOBAR')
        self.assertEqual(self.service.get_action.return_value, res)
        self.service.get_action.assert_called_once_with('FOOBAR')

    def test_update_action(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.update_action('FAKE_ID', status='CANCELLED')
        self.assertEqual(self.service.update_action.return_value, res)
        self.service.update_action.assert_called_once_with(
            'FAKE_ID', status='CANCELLED')

    def test_events(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.events(foo='bar')
        self.assertEqual(self.service.events.return_value, res)
        self.service.events.assert_called_once_with(foo='bar')

    def test_get_event(self, mock_conn):
        mock_conn.return_value = self.conn
        sc = client.Client()

        res = sc.get_event('FOOBAR')
        self.assertEqual(self.service.get_event.return_value, res)
        self.service.get_event.assert_called_once_with('FOOBAR')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import io
from unittest import mock

from openstack import exceptions as sdk_exc
from osc_lib import exceptions as exc
from osc_lib import utils

from senlinclient.tests.unit.v1 import fakes
from senlinclient.v1 import profile as osc_profile


class TestProfile(fakes.TestClusteringv1):
    def setUp(self):
        super(TestProfile, self).setUp()
        self.mock_client = self.app.client_manager.clustering


class TestProfileShow(TestProfile):
    response = {"profile": {
        "created_at": "2015-03-01T14:28:25",
        "domain": 'false',
        "id": "7fa885cd-fa39-4531-a42d-780af95c84a4",
        "metadata": {},
        "name": "test_prof1",
        "project": "42d9e9663331431f97b75e25136307ff",
        "spec": {
            "disable_rollback": 'false',
            "environment": {
                "resource_registry": {
                    "os.heat.server": "OS::Heat::Server"
                }
            },
            "files": {
                "file:///opt/stack/senlin/examples/profiles/test_script.sh":
                    "#!/bin/bash\n\necho \"this is a test script file\"\n"
            },
            "parameters": {},
            "template": {
                "heat_template_version": "2014-10-16",
                "outputs": {
                    "result": {
                        "value": {
                            "get_attr": [
                                "random",
                                "value"
                            ]
                        }
                    }
                },
                "parameters": {
                    "file": {
                        "default": {
                            "get_file": "file:///opt/stack/senlin/"
                                        "examples/profiles/test_script.sh"
                        },
                        "type": "string"
                    }
                },
                "resources": {
                    "random": {
                        "properties": {
                            "length": 64
                        },
                        "type": "OS::Heat::RandomString"
                    }
                },
                "timeout": 60
            },
            "type": "os.heat.stack",
            "version": "1.0"
        },
        "type": "os.heat.stack-1.0",
        "updated_at": 'null',
        "user": "5e5bf8027826429c96af157f68dc9072"
    }}

    def setUp(self):
        super(TestProfileShow, self).setUp()
        self.cmd = osc_profile.ShowProfile(self.app, None)
        fake_profile = mock.Mock(
            created_at="2015-03-01T14:28:25",
            domain_id=None,
            id="7fa885cd-fa39-4531-a42d-780af95c84a4",
            metadata={},
            project_id="42d9e9663331431f97b75e25136307ff",
            spec={"foo": 'bar'},
            type="os.heat.stack-1.0",
            updated_at=None,
            user_id="5e5bf8027826429c96af157f68dc9072"
        )
        fake_profile.name = "test_prof1"
        fake_profile.to_dict = mock.Mock(return_value={})
        self.mock_client.get_profile = mock.Mock(return_value=fake_profile)
        utils.get_dict_properties = mock.Mock(return_value='')

    def test_profile_show(self):
        arglist = ['my_profile']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.get_profile.assert_called_with('my_profile')
        profile = self.mock_client.get_profile('my_profile')
        self.assertEqual("42d9e9663331431f97b75e25136307ff",
                         profile.project_id)
        self.assertEqual("7fa885cd-fa39-4531-a42d-780af95c84a4", profile.id)
        self.assertEqual({}, profile.metadata)
        self.assertEqual("test_prof1", profile.name)
        self.assertEqual("os.heat.stack-1.0", profile.type)

    def test_profile_show_not_found(self):
        arglist = ['my_profile']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.get_profile.side_effect = sdk_exc.ResourceNotFound()
        self.assertRaises(
            exc.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestProfileList(TestProfile):

    def setUp(self):
        super(TestProfileList, self).setUp()
        self.cmd = osc_profile.ListProfile(self.app, None)
        fake_profile = mock.Mock(
            created_at="2015-03-01T14:28:25",
            domain_id=None,
            id="7fa885cd-fa39-4531-a42d-780af95c84a4",
            metadata={},
            project_id="42d9e9663331431f97b75e25136307ff",
            spec={"foo": 'bar'},
            type="os.heat.stack-1.0",
            updated_at=None,
            user_id="5e5bf8027826429c96af157f68dc9072"
        )
        fake_profile.name = "test_profile"
        fake_profile.to_dict = mock.Mock(return_value={})
        self.mock_client.profiles = mock.Mock(return_value=[fake_profile])
        self.defaults = {
            'limit': None,
            'marker': None,
            'sort': None,
            'global_project': False,
        }
        self.columns = ['id', 'name', 'type', 'created_at']

    def test_profile_list_defaults(self):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.profiles.assert_called_with(**self.defaults)
        self.assertEqual(self.columns, columns)

    def test_profile_list_full_id(self):
        arglist = ['--full-id']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.profiles.assert_called_with(**self.defaults)
        self.assertEqual(self.columns, columns)

    def test_profile_list_limit(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['limit'] = '3'
        arglist = ['--limit', '3']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.profiles.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)

    def test_profile_list_sort(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['sort'] = 'id:asc'
        arglist = ['--sort', 'id:asc']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.profiles.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)

    def test_profile_list_sort_invalid_key(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['sort'] = 'bad_key'
        arglist = ['--sort', 'bad_key']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.profiles.side_effect = sdk_exc.HttpException()
        self.assertRaises(sdk_exc.HttpException,
                          self.cmd.take_action, parsed_args)

    def test_profile_list_sort_invalid_direction(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['sort'] = 'id:bad_direction'
        arglist = ['--sort', 'id:bad_direction']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.profiles.side_effect = sdk_exc.HttpException()
        self.assertRaises(sdk_exc.HttpException,
                          self.cmd.take_action, parsed_args)

    def test_profile_list_filter(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['name'] = 'my_profile'
        arglist = ['--filter', 'name=my_profile']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.profiles.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)


class TestProfileDelete(TestProfile):

    def setUp(self):
        super(TestProfileDelete, self).setUp()
        self.cmd = osc_profile.DeleteProfile(self.app, None)
        self.mock_client.delete_profile = mock.Mock()

    def test_profile_delete(self):
        arglist = ['profile1', 'profile2', 'profile3']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.delete_profile.assert_has_calls(
            [mock.call('profile1', False), mock.call('profile2', False),
             mock.call('profile3', False)]
        )

    def test_profile_delete_force(self):
        arglist = ['profile1', 'profile2', 'profile3', '--force']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.delete_profile.assert_has_calls(
            [mock.call('profile1', False), mock.call('profile2', False),
             mock.call('profile3', False)]
        )

    def test_profile_delete_not_found(self):
        arglist = ['my_profile']
        self.mock_client.delete_profile.side_effect = sdk_exc.ResourceNotFound
        parsed_args = self.check_parser(self.cmd, arglist, [])
        error = self.assertRaises(exc.CommandError, self.cmd.take_action,
                                  parsed_args)
        self.assertIn('Failed to delete 1 of the 1 specified profile(s).',
                      str(error))

    def test_profile_delete_one_found_one_not_found(self):
        arglist = ['profile1', 'profile2']
        self.mock_client.delete_profile.side_effect = (
            [None, sdk_exc.ResourceNotFound]
        )
        parsed_args = self.check_parser(self.cmd, arglist, [])
        error = self.assertRaises(exc.CommandError,
                                  self.cmd.take_action, parsed_args)
        self.mock_client.delete_profile.assert_has_calls(
            [mock.call('profile1', False), mock.call('profile2', False)]
        )
        self.assertEqual('Failed to delete 1 of the 2 specified profile(s).',
                         str(error))

    @mock.patch('sys.stdin', spec=io.StringIO)
    def test_profile_delete_prompt_yes(self, mock_stdin):
        arglist = ['my_profile']
        mock_stdin.isatty.return_value = True
        mock_stdin.readline.return_value = 'y'
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        mock_stdin.readline.assert_called_with()
        self.mock_client.delete_profile.assert_called_with('my_profile',
                                                           False)

    @mock.patch('sys.stdin', spec=io.StringIO)
    def test_profile_delete_prompt_no(self, mock_stdin):
        arglist = ['my_profile']
        mock_stdin.isatty.return_value = True
        mock_stdin.readline.return_value = 'n'
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        mock_stdin.readline.assert_called_with()
        self.mock_client.delete_profile.assert_not_called()


class TestProfileCreate(TestProfile):

    spec_path = 'senlinclient/tests/test_specs/nova_server.yaml'

    def setUp(self):
        super(TestProfileCreate, self).setUp()
        self.cmd = osc_profile.CreateProfile(self.app, None)
        fake_profile = mock.Mock(
            created_at="2015-03-01T14:28:25",
            domain_id=None,
            id="7fa885cd-fa39-4531-a42d-780af95c84a4",
            metadata={},
            project_id="42d9e9663331431f97b75e25136307ff",
            spec={"foo": 'bar'},
            type="os.heat.stack-1.0",
            updated_at=None,
            user_id="5e5bf8027826429c96af157f68dc9072"
        )
        fake_profile.name = "test_profile"
        fake_profile.to_dict = mock.Mock(return_value={})
        self.mock_client.create_profile = mock.Mock(return_value=fake_profile)
        self.mock_client.get_profile = mock.Mock(return_value=fake_profile)
        utils.get_dict_properties = mock.Mock(return_value='')
        self.defaults = {
            "spec": {
                "version": 1.0,
                "type": "os.nova.server",
                "properties": {
                    "flavor": 1,
                    "name": "cirros_server",
                    "image": "cirros-0.3.4-x86_64-uec"
                },
            },
            "name": "my_profile",
            "metadata": {}
        }

    def test_profile_create_defaults(self):
        arglist = ['my_profile', '--spec-file', self.spec_path]
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        self.mock_client.create_profile.assert_called_with(**self.defaults)

    def test_profile_create_metadata(self):
        arglist = ['my_profile', '--spec-file', self.spec_path,
                   '--metadata', 'key1=value1']
        kwargs = copy.deepcopy(self.defaults)
        kwargs['metadata'] = {'key1': 'value1'}
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.create_profile.assert_called_with(**kwargs)


class TestProfileUpdate(TestProfile):

    def setUp(self):
        super(TestProfileUpdate, self).setUp()
        self.cmd = osc_profile.UpdateProfile(self.app, None)
        fake_profile = mock.Mock(
            created_at="2015-03-01T14:28:25",
            domain_id=None,
            id="7fa885cd-fa39-4531-a42d-780af95c84a4",
            metadata={},
            project_id="42d9e9663331431f97b75e25136307ff",
            spec={"foo": 'bar'},
            type="os.heat.stack-1.0",
            updated_at=None,
            user_id="5e5bf8027826429c96af157f68dc9072"
        )
        fake_profile.name = "test_profile"
        fake_profile.to_dict = mock.Mock(return_value={})
        self.mock_client.update_profile = mock.Mock(return_value=fake_profile)
        self.mock_client.get_profile = mock.Mock(return_value=fake_profile)
        self.mock_client.find_profile = mock.Mock(return_value=fake_profile)
        utils.get_dict_properties = mock.Mock(return_value='')

    def test_profile_update_defaults(self):
        arglist = ['--name', 'new_profile', '--metadata', 'nk1=nv1;nk2=nv2',
                   'e3057c77']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        defaults = {
            "name": "new_profile",
            "metadata": {
                "nk1": "nv1",
                "nk2": "nv2",
            }
        }

        self.cmd.take_action(parsed_args)

        self.mock_client.update_profile.assert_called_with(
            "7fa885cd-fa39-4531-a42d-780af95c84a4", **defaults)

    def test_profile_update_not_found(self):
        arglist = ['--name', 'new_profile', '--metadata', 'nk1=nv1;nk2=nv2',
                   'c6b8b252']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.find_profile.return_value = None
        error = self.assertRaises(
            exc.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn('Profile not found: c6b8b252', str(error))


class TestProfileValidate(TestProfile):

    spec_path = 'senlinclient/tests/test_specs/nova_server.yaml'
    defaults = {
        "spec": {
            "version": 1.0,
            "type": "os.nova.server",
            "properties": {
                "flavor": 1,
                "name": "cirros_server",
                "image": "cirros-0.3.4-x86_64-uec"
            },
        }
    }

    def setUp(self):
        super(TestProfileValidate, self).setUp()
        self.cmd = osc_profile.ValidateProfile(self.app, None)
        fake_profile = mock.Mock(
            created_at=None,
            domain_id=None,
            id=None,
            metadata={},
            project_id="42d9e9663331431f97b75e25136307ff",
            spec={"foo": 'bar'},
            type="os.heat.stack-1.0",
            updated_at=None,
            user_id="5e5bf8027826429c96af157f68dc9072"
        )
        fake_profile.name = "test_profile"
        fake_profile.to_dict = mock.Mock(return_value={})
        self.mock_client.validate_profile = mock.Mock(
            return_value=fake_profile)
        utils.get_dict_properties = mock.Mock(return_value='')

    def test_profile_validate(self):
        arglist = ['--spec-file', self.spec_path]
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.validate_profile.assert_called_with(**self.defaults)

        profile = self.mock_client.validate_profile(**self.defaults)

        self.assertEqual("42d9e9663331431f97b75e25136307ff",
                         profile.project_id)
        self.assertEqual("5e5bf8027826429c96af157f68dc9072", profile.user_id)
        self.assertIsNone(profile.id)
        self.assertEqual({}, profile.metadata)
        self.assertEqual("test_profile", profile.name)
        self.assertEqual("os.heat.stack-1.0", profile.type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlinclient.tests.functional import base


class ActionTest(base.OpenStackClientTestBase):
    """Test for actions"""

    def test_action_list(self):
        result = self.openstack('cluster action list')
        action_list = self.parser.listing(result)
        self.assertTableStruct(action_list,
                               ['id', 'name', 'action', 'status',
                                'target_id', 'depends_on', 'cluster_id',
                                'depended_by', 'created_at'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Clustering v1 receiver action implementations"""

import logging
import sys

from openstack import exceptions as sdk_exc
from osc_lib.command import command
from osc_lib import exceptions as exc
from osc_lib import utils

from senlinclient.common.i18n import _
from senlinclient.common import utils as senlin_utils


class ListReceiver(command.Lister):
    """List receivers that meet the criteria."""

    log = logging.getLogger(__name__ + ".ListReceiver")

    def get_parser(self, prog_name):
        parser = super(ListReceiver, self).get_parser(prog_name)
        parser.add_argument(
            '--filters',
            metavar='<"key1=value1;key2=value2...">',
            help=_("Filter parameters to apply on returned receivers. "
                   "This can be specified multiple times, or once with "
                   "parameters separated by a semicolon. The valid filter "
                   "keys are: ['name', 'type', 'action', 'cluster_id', "
                   "'user_id']"),
            action='append'
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            help=_('Limit the number of receivers returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Only return receivers that appear after the given '
                   'receiver ID')
        )
        parser.add_argument(
            '--sort',
            metavar='<key>[:<direction>]',
            help=_("Sorting option which is a string containing a list of "
                   "keys separated by commas. Each key can be optionally "
                   "appended by a sort direction (:asc or :desc). The valid "
                   "sort keys are: ['name', 'type', 'action', 'cluster_id', "
                   "'created_at']")
        )
        parser.add_argument(
            '--global-project',
            default=False,
            action="store_true",
            help=_('Indicate that the list should include receivers from'
                   ' all projects. This option is subject to access policy '
                   'checking. Default is False')
        )
        parser.add_argument(
            '--full-id',
            default=False,
            action="store_true",
            help=_('Print full IDs in list')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        senlin_client = self.app.client_manager.clustering

        columns = ['id', 'name', 'type', 'cluster_id', 'action', 'created_at']
        queries = {
            'limit': parsed_args.limit,
            'marker': parsed_args.marker,
            'sort': parsed_args.sort,
            'global_project': parsed_args.global_project,
        }

        if parsed_args.filters:
            queries.update(senlin_utils.format_parameters(parsed_args.filters))

        receivers = senlin_client.receivers(**queries)
        formatters = {}
        if parsed_args.global_project:
            columns.append('project_id')
            columns.append('user_id')
        if not parsed_args.full_id:
            formatters = {
                'id': lambda x: x[:8],
                'cluster_id': lambda x: x[:8] if x else None,
            }
            if 'project_id' in columns:
                formatters['project_id'] = lambda x: x[:8]
                formatters['user_id'] = lambda x: x[:8]

        return (
            columns,
            (utils.get_item_properties(r, columns, formatters=formatters)
             for r in receivers)
        )


class ShowReceiver(command.ShowOne):
    """Show the receiver details."""

    log = logging.getLogger(__name__ + ".ShowReceiver")

    def get_parser(self, prog_name):
        parser = super(ShowReceiver, self).get_parser(prog_name)
        parser.add_argument(
            'receiver',
            metavar='<receiver>',
            help=_('Name or ID of the receiver to show')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        senlin_client = self.app.client_manager.clustering
        return _show_receiver(senlin_client, parsed_args.receiver)


def _show_receiver(senlin_client, receiver_id):
    try:
        receiver = senlin_client.get_receiver(receiver_id)
    except sdk_exc.ResourceNotFound:
        raise exc.CommandError(_('Receiver not found: %s') % receiver_id)

    formatters = {
        'actor': senlin_utils.json_formatter,
        'params': senlin_utils.json_formatter,
        'channel': senlin_utils.json_formatter,
    }
    data = receiver.to_dict()
    columns = sorted(data.keys())
    return columns, utils.get_dict_properties(data, columns,
                                              formatters=formatters)


class CreateReceiver(command.ShowOne):
    """Create a receiver."""

    log = logging.getLogger(__name__ + ".CreateReceiver")

    def get_parser(self, prog_name):
        parser = super(CreateReceiver, self).get_parser(prog_name)
        parser.add_argument(
            '--type',
            metavar='<type>',
            default='webhook',
            help=_('Type of the receiver to create. Receiver type can be '
                   '"webhook" or "message". Default to "webhook".')
        )
        parser.add_argument(
            '--params',
            metavar='<"key1=value1;key2=value2...">',
            help=_('A dictionary of parameters that will be passed to target '
                   'action when the receiver is triggered'),
            action='append'
        )
        parser.add_argument(
            '--cluster',
            metavar='<cluster>',
            help=_('Targeted cluster for this receiver. Required if '
                   'receiver type is webhook')
        )
        parser.add_argument(
            '--action',
            metavar='<action>',
            help=_('Name or ID of the targeted action to be triggered. '
                   'Required if receiver type is webhook')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the receiver to create')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        if parsed_args.type == 'webhook':
            if (not parsed_args.cluster or not parsed_args.action):
                msg = _('cluster and action parameters are required to create '
                        'webhook type of receiver.')
                raise exc.CommandError(msg)

        senlin_client = self.app.client_manager.clustering
        params = {
            'name': parsed_args.name,
            'type': parsed_args.type,
            'cluster_id': parsed_args.cluster,
            'action': parsed_args.action,
            'params': senlin_utils.format_parameters(parsed_args.params)
        }

        receiver = senlin_client.create_receiver(**params)
        return _show_receiver(senlin_client, receiver.id)


class UpdateReceiver(command.ShowOne):
    """Update a receiver."""

    log = logging.getLogger(__name__ + ".UpdateReceiver")

    def get_parser(self, prog_name):
        parser = super(UpdateReceiver, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the receiver to create')
        )
        parser.add_argument(
            '--action',
            metavar='<action>',
            help=_('Name or ID of the targeted action to be triggered. '
                   'Required if receiver type is webhook')
        )
        parser.add_argument(
            '--params',
            metavar='<"key1=value1;key2=value2...">',
            help=_('A dictionary of parameters that will be passed to target '
                   'action when the receiver is triggered'),
            action='append'
        )
        parser.add_argument(
            'receiver',
            metavar='<receiver>',
            help=_('Name or ID of receiver(s) to update')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        senlin_client = self.app.client_manager.clustering
        params = {
            'name': parsed_args.name,
            'params': senlin_utils.format_parameters(parsed_args.params)
        }
        if parsed_args.action:
            params['action'] = parsed_args.action

        receiver = senlin_client.find_receiver(parsed_args.receiver)
        if receiver is None:
            raise exc.CommandError(_('Receiver not found: %s') %
                                   parsed_args.receiver)
        senlin_client.update_receiver(receiver.id, **params)
        return _show_receiver(senlin_client, receiver_id=receiver.id)


class DeleteReceiver(command.Command):
    """Delete receiver(s)."""

    log = logging.getLogger(__name__ + ".DeleteReceiver")

    def get_parser(self, prog_name):
        parser = super(DeleteReceiver, self).get_parser(prog_name)
        parser.add_argument(
            'receiver',
            metavar='<receiver>',
            nargs='+',
            help=_('Name or ID of receiver(s) to delete')
        )
        parser.add_argument(
            '--force',
            action='store_true',
            help=_('Skip yes/no prompt (assume yes)')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        senlin_client = self.app.client_manager.clustering

        try:
            if not parsed_args.force and sys.stdin.isatty():
                sys.stdout.write(
                    _("Are you sure you want to delete this receiver(s)"
                      " [y/N]?"))
                sys.stdout.flush()
                prompt_response = sys.stdin.readline().lower()
                if not prompt_response.startswith('y'):
                    return
        except KeyboardInterrupt:  # Ctrl-c
            self.log.info('Ctrl-c detected.')
            return
        except EOFError:  # Ctrl-d
            self.log.info('Ctrl-d detected')
            return

        failure_count = 0

        for rid in parsed_args.receiver:
            try:
                senlin_client.delete_receiver(rid, False)
            except Exception as ex:
                failure_count += 1
                print(ex)
        if failure_count:
            raise exc.CommandError(_('Failed to delete %(count)s of the '
                                     '%(total)s specified receiver(s).') %
                                   {'count': failure_count,
                                   'total': len(parsed_args.receiver)})
        print('Receiver deleted: %s' % parsed_args.receiver)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import io
from unittest import mock

from openstack import exceptions as sdk_exc
from osc_lib import exceptions as exc
from osc_lib import utils

from senlinclient.common.i18n import _
from senlinclient.tests.unit.v1 import fakes
from senlinclient.v1 import receiver as osc_receiver


class TestReceiver(fakes.TestClusteringv1):
    def setUp(self):
        super(TestReceiver, self).setUp()
        self.mock_client = self.app.client_manager.clustering


class TestReceiverList(TestReceiver):
    columns = ['id', 'name', 'type', 'cluster_id', 'action', 'created_at']
    defaults = {
        'global_project': False,
        'marker': None,
        'limit': None,
        'sort': None,
    }

    def setUp(self):
        super(TestReceiverList, self).setUp()
        self.cmd = osc_receiver.ListReceiver(self.app, None)
        fake_receiver = mock.Mock(
            action="CLUSTER_SCALE_OUT",
            actor={},
            channel={
                "alarm_url": "http://node1:8778/v1/webhooks/e03dd2e5-8f2e-4ec1"
                             "-8c6a-74ba891e5422/trigger?V=1&count=1"
            },
            cluster_id="ae63a10b-4a90-452c-aef1-113a0b255ee3",
            created_at="2015-06-27T05:09:43",
            domain_id="Default",
            id="573aa1ba-bf45-49fd-907d-6b5d6e6adfd3",
            name="cluster_inflate",
            params={"count": "1"},
            project_id="6e18cc2bdbeb48a5b3cad2dc499f6804",
            type="webhook",
            updated_at=None,
            user_id="b4ad2d6e18cc2b9c48049f6dbe8a5b3c"
        )
        fake_receiver.to_dict = mock.Mock(return_value={})
        self.mock_client.receivers = mock.Mock(return_value=[fake_receiver])

    def test_receiver_list_defaults(self):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.receivers.assert_called_with(**self.defaults)
        self.assertEqual(self.columns, columns)

    def test_receiver_list_full_id(self):
        arglist = ['--full-id']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.receivers.assert_called_with(**self.defaults)
        self.assertEqual(self.columns, columns)

    def test_receiver_list_limit(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['limit'] = '3'
        arglist = ['--limit', '3']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.receivers.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)

    def test_receiver_list_sort(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['sort'] = 'name:asc'
        arglist = ['--sort', 'name:asc']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.receivers.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)

    def test_receiver_list_sort_invalid_key(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['sort'] = 'bad_key'
        arglist = ['--sort', 'bad_key']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.receivers.side_effect = sdk_exc.HttpException()
        self.assertRaises(sdk_exc.HttpException,
                          self.cmd.take_action, parsed_args)

    def test_receiver_list_sort_invalid_direction(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['sort'] = 'name:bad_direction'
        arglist = ['--sort', 'name:bad_direction']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.receivers.side_effect = sdk_exc.HttpException()
        self.assertRaises(sdk_exc.HttpException,
                          self.cmd.take_action, parsed_args)

    def test_receiver_list_filter(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['name'] = 'my_receiver'
        arglist = ['--filter', 'name=my_receiver']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.receivers.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)

    def test_receiver_list_marker(self):
        kwargs = copy.deepcopy(self.defaults)
        kwargs['marker'] = 'a9448bf6'
        arglist = ['--marker', 'a9448bf6']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.receivers.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)


class TestReceiverShow(TestReceiver):

    def setUp(self):
        super(TestReceiverShow, self).setUp()
        self.cmd = osc_receiver.ShowReceiver(self.app, None)
        fake_receiver = mock.Mock(
            action="CLUSTER_SCALE_OUT",
            actor={},
            channel={
                "alarm_url": "http://node1:8778/v1/webhooks/e03dd2e5-8f2e-4ec1"
                             "-8c6a-74ba891e5422/trigger?V=1&count=1"
            },
            cluster_id="ae63a10b-4a90-452c-aef1-113a0b255ee3",
            created_at="2015-06-27T05:09:43",
            domain_id="Default",
            id="573aa1ba-bf45-49fd-907d-6b5d6e6adfd3",
            name="cluster_inflate",
            params={"count": "1"},
            project_id="6e18cc2bdbeb48a5b3cad2dc499f6804",
            type="webhook",
            updated_at=None,
            user_id="b4ad2d6e18cc2b9c48049f6dbe8a5b3c"
        )
        fake_receiver.to_dict = mock.Mock(return_value={})
        self.mock_client.get_receiver = mock.Mock(return_value=fake_receiver)

    def test_receiver_show(self):
        arglist = ['my_receiver']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.get_receiver.assert_called_with('my_receiver')

    def test_receiver_show_not_found(self):
        arglist = ['my_receiver']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.get_receiver.side_effect = sdk_exc.ResourceNotFound()
        error = self.assertRaises(exc.CommandError, self.cmd.take_action,
                                  parsed_args)
        self.assertEqual('Receiver not found: my_receiver', str(error))


class TestReceiverCreate(TestReceiver):

    args = {
        "action": "CLUSTER_SCALE_OUT",
        "cluster_id": "my_cluster",
        "name": "my_receiver",
        "params": {
            "count": "1"
        },
        "type": "webhook"
    }

    def setUp(self):
        super(TestReceiverCreate, self).setUp()
        self.cmd = osc_receiver.CreateReceiver(self.app, None)
        fake_receiver = mock.Mock(
            action="CLUSTER_SCALE_OUT",
            actor={},
            channel={
                "alarm_url": "http://node1:8778/v1/webhooks/e03dd2e5-8f2e-4ec1"
                             "-8c6a-74ba891e5422/trigger?V=1&count=1"
            },
            cluster_id="ae63a10b-4a90-452c-aef1-113a0b255ee3",
            created_at="2015-06-27T05:09:43",
            domain_id="Default",
            id="573aa1ba-bf45-49fd-907d-6b5d6e6adfd3",
            name="cluster_inflate",
            params={"count": "1"},
            project_id="6e18cc2bdbeb48a5b3cad2dc499f6804",
            type="webhook",
            updated_at=None,
            user_id="b4ad2d6e18cc2b9c48049f6dbe8a5b3c"
        )
        fake_receiver.to_dict = mock.Mock(return_value={})
        self.mock_client.create_receiver = mock.Mock(
            return_value=fake_receiver)
        self.mock_client.get_receiver = mock.Mock(
            return_value=fake_receiver)

    def test_receiver_create_webhook(self):
        arglist = ['my_receiver', '--action', 'CLUSTER_SCALE_OUT',
                   '--cluster', 'my_cluster', '--params', 'count=1',
                   '--type', 'webhook']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.create_receiver.assert_called_with(**self.args)

    def test_receiver_create_webhook_failed(self):
        arglist = ['my_receiver', '--action', 'CLUSTER_SCALE_OUT',
                   '--params', 'count=1', '--type', 'webhook']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        error = self.assertRaises(exc.CommandError, self.cmd.take_action,
                                  parsed_args)
        self.assertIn(_('cluster and action parameters are required to create '
                        'webhook type of receiver'), str(error))

    def test_receiver_create_non_webhook(self):
        arglist = ['my_receiver', '--params', 'count=1',
                   '--type', 'foo']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        args = copy.deepcopy(self.args)
        args['type'] = 'foo'
        args['cluster_id'] = None
        args['action'] = None
        self.mock_client.create_receiver.assert_called_with(**args)


class TestReceiverUpdate(TestReceiver):
    args = {
        "action": "CLUSTER_SCALE_OUT",
        "name": "test_receiver",
        "params": {
            "count": "2"
        },
    }

    def setUp(self):
        super(TestReceiverUpdate, self).setUp()
        self.cmd = osc_receiver.UpdateReceiver(self.app, None)
        fake_receiver = mock.Mock(
            action="CLUSTER_SCALE_IN",
            actor={},
            channel={
                "alarm_url": "http://node1:8778/v1/webhooks/e03dd2e5-8f2e-4ec1"
                             "-8c6a-74ba891e5422/trigger?V=1&count=1"
            },
            created_at="2015-06-27T05:09:43",
            domain_id="Default",
            id="573aa1ba-bf45-49fd-907d-6b5d6e6adfd3",
            params={"count": "1"},
            project_id="6e18cc2bdbeb48a5b3cad2dc499f6804",
            updated_at=None,
            user_id="b4ad2d6e18cc2b9c48049f6dbe8a5b3c"
        )
        fake_receiver.name = "cluster_inflate"
        fake_receiver.action = "CLUSTER_SCALE_IN"
        fake_receiver.params = {"count": "1"}
        fake_receiver.to_dict = mock.Mock(return_value={})
        self.mock_client.update_receiver = mock.Mock(
            return_value=fake_receiver)
        self.mock_client.get_receiver = mock.Mock(return_value=fake_receiver)
        self.mock_client.find_receiver = mock.Mock(return_value=fake_receiver)
        utils.get_dict_properties = mock.Mock(return_value='')

    def test_receiver_update_defaults(self):
        arglist = ['--name', 'test_receiver', '--action', 'CLUSTER_SCALE_OUT',
                   '--params', 'count=2', '573aa1ba']
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        self.mock_client.update_receiver.assert_called_with(
            "573aa1ba-bf45-49fd-907d-6b5d6e6adfd3", **self.args)

    def test_receiver_update_not_found(self):
        arglist = ['--name', 'test_receiver', '--action', 'CLUSTER_SCALE_OUT',
                   '--params', 'count=2', '573aa1b2']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.find_receiver.return_value = None
        error = self.assertRaises(exc.CommandError,
                                  self.cmd.take_action,
                                  parsed_args)
        self.assertIn('Receiver not found: 573aa1b2', str(error))


class TestReceiverDelete(TestReceiver):

    def setUp(self):
        super(TestReceiverDelete, self).setUp()
        self.cmd = osc_receiver.DeleteReceiver(self.app, None)
        self.mock_client.delete_receiver = mock.Mock()

    def test_receiver_delete(self):
        arglist = ['receiver1', 'receiver2', 'receiver3']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.delete_receiver.assert_has_calls(
            [mock.call('receiver1', False), mock.call('receiver2', False),
             mock.call('receiver3', False)]
        )

    def test_receiver_delete_force(self):
        arglist = ['receiver1', 'receiver2', 'receiver3', '--force']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.delete_receiver.assert_has_calls(
            [mock.call('receiver1', False), mock.call('receiver2', False),
             mock.call('receiver3', False)]
        )

    def test_receiver_delete_not_found(self):
        arglist = ['my_receiver']
        self.mock_client.delete_receiver.side_effect = (
            sdk_exc.ResourceNotFound)
        parsed_args = self.check_parser(self.cmd, arglist, [])
        error = self.assertRaises(exc.CommandError, self.cmd.take_action,
                                  parsed_args)
        self.assertIn('Failed to delete 1 of the 1 specified receiver(s).',
                      str(error))

    def test_receiver_delete_one_found_one_not_found(self):
        arglist = ['receiver1', 'receiver2']
        self.mock_client.delete_receiver.side_effect = (
            [None, sdk_exc.ResourceNotFound]
        )
        parsed_args = self.check_parser(self.cmd, arglist, [])
        error = self.assertRaises(exc.CommandError,
                                  self.cmd.take_action, parsed_args)
        self.mock_client.delete_receiver.assert_has_calls(
            [mock.call('receiver1', False), mock.call('receiver2', False)]
        )
        self.assertEqual('Failed to delete 1 of the 2 specified receiver(s).',
                         str(error))

    @mock.patch('sys.stdin', spec=io.StringIO)
    def test_receiver_delete_prompt_yes(self, mock_stdin):
        arglist = ['my_receiver']
        mock_stdin.isatty.return_value = True
        mock_stdin.readline.return_value = 'y'
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        mock_stdin.readline.assert_called_with()
        self.mock_client.delete_receiver.assert_called_with('my_receiver',
                                                            False)

    @mock.patch('sys.stdin', spec=io.StringIO)
    def test_receiver_delete_prompt_no(self, mock_stdin):
        arglist = ['my_receiver']
        mock_stdin.isatty.return_value = True
        mock_stdin.readline.return_value = 'n'
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.cmd.take_action(parsed_args)

        mock_stdin.readline.assert_called_with()
        self.mock_client.delete_receiver.assert_not_called()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from senlinclient.tests.functional import base


class HelpTest(base.OpenStackClientTestBase):
    """Test for help commands"""

    def test_help_cmd(self):
        help_text = self.openstack('help cluster list')
        lines = help_text.split('\n')
        self.assertFirstLineStartsWith(lines, 'usage: openstack cluster list')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from osc_lib.command import command


class RawFormat(command.ShowOne):

    def produce_output(self, parsed_args, column_names, data):
        if data is None:
            return

        self.formatter.emit_one(column_names, data,
                                self.app.stdout, parsed_args)


class JsonFormat(RawFormat):

    @property
    def formatter_default(self):
        return 'json'


class YamlFormat(RawFormat):

    @property
    def formatter_default(self):
        return 'yaml'


class ShellFormat(RawFormat):

    @property
    def formatter_default(self):
        return 'shell'


class ValueFormat(RawFormat):

    @property
    def formatter_default(self):
        return 'value'
